<?php

namespace Drupal\migrate_boost\Commands;

use Consolidation\AnnotatedCommand\AnnotationData;
use Drupal\migrate_boost\MigrateBoost;
use Drush\Commands\DrushCommands;
use Symfony\Component\Console\Input\InputInterface;

/**
 * Migrate Boost drush commands.
 *
 * See these files for an example of injecting Drupal services:
 *   - http://cgit.drupalcode.org/devel/tree/src/Commands/DevelCommands.php
 *   - http://cgit.drupalcode.org/devel/tree/drush.services.yml.
 */
class MigrateBoostCommands extends DrushCommands {

  /**
   * Resets migrate booster and implementation cache.
   *
   * @command migrate:booster:reset
   *
   * @validate-module-enabled migrate_boost
   * @aliases mbr,migrate-booster-reset
   */
  public function boosterReset() {
    MigrateBoost::reset();
  }

  /**
   * Enables migrate booster and implementation cache.
   *
   * @command migrate:booster:enable
   *
   * @validate-module-enabled migrate_boost
   * @aliases mbe,migrate-booster-enable
   */
  public function boosterEnable() {
    MigrateBoost::enable();
  }

  /**
   * Interact with drush on init.
   *
   * @hook init
   */
  public function initCommand(InputInterface $input, AnnotationData $annotationData) {
    // Skip when bootstrap level is low (e.g. drush cr)
    if (!\Drupal::hasContainer()) {
      return;
    }
    MigrateBoost::bootDrush($input, $annotationData);
  }

}
